/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __KIPF_DEFS_H__
#define __KIPF_DEFS_H__

/* Version of this driver */
#define	KIPF_MAJOR_VERSION			2
#define	KIPF_MINOR_VERSION			0

/* Out 32 bit ptr type */
typedef	unsigned long KIPF_PTR;

/*	Max count of buffers for request */
#define MAX_RW_BUFFERS_COUNT		64

/* rules for set_default_traffic_rule() */
#define TRAFFIC_BLOCKED				0
#define TRAFFIC_ALLOWED				1

#define	KIPF_MAGIC_CHECK			0xbeef

/* Magic numbers */
#define	KIPF_MAGIC_FREE				0x0
#define	KIPF_MAGIC_IFACE			0x1
#define	KIPF_MAGIC_PACKET			0x2

/* Macro for magic number computation */
#define	KIPF_MAGIC(x)				((unsigned int)(((unsigned short)((unsigned int)(x) & 0xffff)) | \
	((unsigned int)((unsigned short)((unsigned int)(KIPF_MAGIC_CHECK) & 0xffff))) << 16))

/*	Maximum lenghth of interface name */
#define KIPF_MAX_NSIZE				128

/*	Maximum length of kipf interface hardware address */
#define KIPF_HW_ADDR_LEN			6

/*	Max length of version string */
#define KIPF_VERSION_STRING_LEN		32

#pragma pack(1)

/*
 *	header
 */
struct header {
	unsigned int magic;					/*	magic number	*/
	unsigned int len;					/*	length of buffer */
};

/*
 *	kipf_iface
 */
struct kipf_iface {
	struct header header;				/*	struct for packet recognizing	*/
	unsigned int ifindex;			/*	interface index */
	char name[KIPF_MAX_NSIZE];			/*	name of interface	*/
	char id[KIPF_MAX_NSIZE];			/*	id of interface	*/
	char hw_addr[KIPF_HW_ADDR_LEN];		/*	hardware address */	
	unsigned int mtu;					/*	Maximum transmission unit	*/

	unsigned int flags;						
#define KIPF_IFACE_UP	0x1				/* interface is up		*/

	unsigned int hw_type;				
#define KIPF_HW_TYPE_UNK	0x0			/* unknown	*/
#define KIPF_HW_TYPE_ETH	0x1			/* ethernet	*/
#define KIPF_HW_TYPE_PPP	0x2			/* ppp	*/
#define KIPF_HW_TYPE_VPN	0x3			/* vpn	*/
};

/*
 *	kipf_packet
 */
struct kipf_packet {
	struct header header;				/*	struct for packet recognizing	*/
	unsigned int id;				/*	packet id	*/
	unsigned int ifindex;			/*	ifindex of interface where packet was received/sent	*/
	unsigned int received;				/*	packet was received ? */
	unsigned int len;					/*	total length of received/sent packet with hw header	*/	
	unsigned int data_len;				/*	count of bytes copied into data */
	char data[1];						/*	packet data */
};

/*
 *	kipf_route_info
 */
struct kipf_route_info {
	unsigned int next_hop;
	unsigned int ifindex;
};

/*
 *	kipf_action
 */
struct kipf_action {

	unsigned int type;							/*	type of action	*/
#define	KIPF_ACTION_NULL			0x0
#define	KIPF_ACTION_ACCEPT			0x1
#define	KIPF_ACTION_DROP			0x2
#define	KIPF_ACTION_COMPLETE		0x3
#define	KIPF_ACTION_SEND_OUT		0x4
#define	KIPF_ACTION_SEND_IN			0x5

	unsigned int id;						/*	packet id	*/
	unsigned int modif_len;						/*	length of modified part of packet	*/
	struct kipf_route_info route_info;
};

/*
 *	kipf_info
 */
struct kipf_info {    		

	/* maximum size of buffer for reading/writing (currently hardcoded to MAX_RW_BUFFER_SIZE) */
	unsigned int max_rw_buffer_size;	

	/*
	 *	version of driver
	 */
	struct {
		unsigned int major;
		unsigned int minor;		
	} api_version;

	char version[KIPF_VERSION_STRING_LEN];
}; 

/*
 *	read_request
 */
struct read_request {

	/* input - count of pointers in header array */
	/* output - count of successfuly readed buffers */
	unsigned int count;

	/* here is stored pointers to kipf_iface or kipf_packet depending on magic number */	
	KIPF_PTR headers[MAX_RW_BUFFERS_COUNT];
};
typedef struct read_request READ_REQ;

/*
 *	write_request
 */
struct write_request {

	/* input - count of pointers in actions and packets array */
	/* output - count of successfuly handled packets */
	unsigned int count;

	struct kipf_action actions[MAX_RW_BUFFERS_COUNT];

	/* here is stored pointers to kipf_packet */	
	KIPF_PTR packets[MAX_RW_BUFFERS_COUNT];
};
typedef struct write_request WRITE_REQ;

/*
 *	ioctl_request
 */
struct ioctl_request {
	struct kipf_action action;
	unsigned char *pkt;
	unsigned int pkt_len;
	/* here is stored pointer to kipf_packet */	
	KIPF_PTR packet;
};
typedef struct ioctl_request IOCTL_REQ;

#pragma pack()

#endif // __KIPF_DEFS_H__
