/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __HDRS_H__
#define __HDRS_H__

#include <linux/types.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <linux/netdevice.h>
#include <linux/if.h>	// dev->flags (IFF_UP, IFF_LOOPBACK ...)
#include <linux/if_ether.h>
#include <linux/if_arp.h>	// dev->type (ARPHRD_ETHER, ARPHRD_LOOPBACK ...)
#include <linux/skbuff.h>
#include <linux/cdev.h>
#include <linux/device.h>
#include <linux/nls.h> // wchar_t
#include <linux/net.h>
#include <linux/syscalls.h>
#include <linux/netfilter.h>
#include <linux/etherdevice.h>
#include <linux/notifier.h>		//	register_netdevice_notifier
#include <linux/inetdevice.h>	//	register_inetaddr_notifier
#include <linux/version.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <linux/tcp.h>
#include <linux/icmp.h>
#include <net/ip.h>
#include <asm/uaccess.h>

//
//	Linux kernel prior version 2.6.22 is not supported
//
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,22))
#error "--- Linux kernel prior version 2.6.22 is not supported ---"
#endif

//
//	Linux kernel 2.6.22 - 2.6.23
//
#if (LINUX_VERSION_CODE == KERNEL_VERSION(2,6,22) || LINUX_VERSION_CODE == KERNEL_VERSION(2,6,23))
#define LINUX_KERNEL_2_6_22_OR_2_6_23
#endif

//
//	Linux kernel 2.6.22 - 2.6.26
//
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,22) && LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,26))
#define LINUX_KERNEL_2_6_22_TILL_2_6_26
#endif

//
//	Linux kernel 2.6.27
//
#if (LINUX_VERSION_CODE == KERNEL_VERSION(2,6,27))
#define LINUX_KERNEL_2_6_27
#endif

#if !defined(LINUX_KERNEL_2_6_22_OR_2_6_23)

#include <net/net_namespace.h>		// init_net
#include <net/netfilter/nf_queue.h>	// nf_reinject

#ifndef NF_IP_PRE_ROUTING
#define NF_IP_PRE_ROUTING NF_INET_PRE_ROUTING
#endif

#ifndef NF_IP_POST_ROUTING
#define NF_IP_POST_ROUTING NF_INET_POST_ROUTING
#endif

#endif

#define MODULE_NAME "kipf"

//
//	timeout in seconds to wait before all traffic is permited 
//
#define START_TIMEOUT 300

//#define USE_BLOCKING_READ

#define USE_SCATTER_GATHER

#include "utils.h"
#include "init.h"
#include "netdev.h"
#include "recv.h"
#include "send.h"
#include "ip_filter.h"
#include "ip_frag.h"
#include "queue.h"
#include "io_dev.h"
#include "proc.h"
#include "cfg.h"
#include "defs.h"
#include "ioctl.h"

#endif	//	__HDRS_H__
