/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"
#include "socket.h"

/**
 *	g_utils_debug_enabled - used in trace function
 */
int g_utils_debug_enabled = 0;

/**
 *	module_unload
 */
#undef __FUNC__
#define __FUNC__ "module_unload()"
void
module_unload(
	void
	)
{	
	TRACE(LOG_INFO, "%s: ==>", __FUNC__);

	//
	//	Close /proc/linipf ...
	//
	proc_close();

	//
	//	Close net device notifier
	//
	netdev_close();

	//
	//	Close raw socket
	//
	socket_close();

	//
	//	Close IO device
	//
	io_dev_close();

	//
	//	Close IP filter - it MUST be closed after other components, because it will free queue lock.
	//
	ip_filter_close();	

	//
	//	Close configuration
	//
	cfg_close();

	TRACE(LOG_INFO, "%s: <==", __FUNC__);
}

/**
 *	module_load
 */
#undef __FUNC__
#define __FUNC__ "module_load()"
int
module_load(
	void
	)
{
	int err = 0;
	
	TRACE(LOG_INFO, "%s: Kerio IP filter ==>", __FUNC__);
	TRACE(LOG_INFO, "%s: Version of KIPF: %d.%d", __FUNC__, KIPF_MAJOR_VERSION, KIPF_MINOR_VERSION);

	//TRACE(LOG_INFO, "%s: LINUX_VERSION_CODE=%d,  KERNEL_VERSION=%d", __FUNC__, LINUX_VERSION_CODE,  KERNEL_VERSION(2,6,22));

#if 0
	{
		struct task_struct *task = current;
		if (task) {
			TRACE(LOG_INFO, "%s: pid=%u", __FUNC__, task->pid);
		}
	}
#endif

	do {					

		//
		//	Load configuration
		//		
		if ((err = cfg_init())) {		
			break;
		}

		//
		//	Initialize IP filter - it MUST be first
		//
		if ((err = ip_filter_init())) {			
			break;
		}		

		//
		//	Initialize and create IO device
		//
		if ((err = io_dev_init())) {		
			break;
		}

		//
		//	Initialize and create raw socket for ip send
		//
		if ((err = socket_init())) {		
			break;
		}

		//
		//	Initialize net device notifier - it will fill main queue with net devices
		//		
		if ((err = netdev_init())) {		
			break;
		}		

		//
		//	Initialize and create /proc/linipf
		//		
		if ((err = proc_init())) {		
			break;
		}

	} while (0);

	//
	//	On error perform rollback action
	//
	if (err) {			
		module_unload();
	} 

	TRACE(LOG_INFO, "%s: err=%d <==", __FUNC__, err);

	return err;
}

module_init(module_load)
module_exit(module_unload)

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Copyright (C) Kerio Technologies s.r.o.");
MODULE_DESCRIPTION("Kerio IP filter.");

