/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"
#include "pkt.h"

/**
 *	io_write_handle_data
 */
#undef __FUNC__
#define __FUNC__ "io_write_handle_data()"
int
io_write_handle_data(		
	struct kipf_action *action,
	unsigned char *pkt,
	unsigned int pkt_len
	)
{
	int err = -EINVAL;

	switch (action->type) {

		case KIPF_ACTION_SEND_OUT:
			{				
				err = pkt_send_out(pkt, pkt_len, action);										
			}
			break;

		case KIPF_ACTION_SEND_IN:
			{								
				err = pkt_send_in(pkt, pkt_len, action);				
			}
			break;
	}

	return err;
}

/**
 *	io_write_handle_packet
 */
#undef __FUNC__
#define __FUNC__ "io_write_handle_packet()"
int
io_write_handle_packet(		
	struct kipf_action *action,
	struct kipf_packet *packet
	)
{			
	int err = -EINVAL;

	switch (action->type) {

		case KIPF_ACTION_ACCEPT:
			{																				
				err = pkt_accept(packet, action);
			}
			break;

		case KIPF_ACTION_DROP:
			{
				err = pkt_drop(packet, action);
			}
			break;

		case KIPF_ACTION_COMPLETE:
			{
				err = pkt_complete(packet, action);																	
			}
			break;

		default:			
			break;
	}

	return err;
}

/**
 *	io_write_from_iovec
 */
#undef __FUNC__
#define __FUNC__ "io_write_from_iovec()"
ssize_t
io_write_from_iovec(			
	struct file *file,
	struct iovec *iv,
	ssize_t len
	)
{	
	u_int32_t i = 0;	
	ssize_t written = 0;
	int err = 0;
	WRITE_REQ writeRequest;	     

	if (len != sizeof(WRITE_REQ)) {
		TRACE(LOG_WARN, "%s: invalid length. (%d)", __FUNC__, len);
		return -EINVAL;
	}

	if (memcpy_fromiovec((unsigned char*) &writeRequest, iv, len)) {
		TRACE(LOG_WARN, "%s: memcpy_fromiovec(): segmentation fault.", __FUNC__);		
		return -EFAULT;
	}	

	for (i = 0; i < writeRequest.count; i++) {
		if (!(err = io_write_handle_packet(&writeRequest.actions[i], (struct kipf_packet*)writeRequest.packets[i]))) {
			written++;
		}
	}	

	writeRequest.count = written;

    return written;
}

