/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"

/**
 *	ip_frag_gather
 */
#undef __FUNC__
#define __FUNC__ "ip_frag_gather()"
struct sk_buff*
ip_frag_gather(
	struct sk_buff *frag_skb
	)
{	
	struct sk_buff *defrag_skb = NULL;
#if !defined(LINUX_KERNEL_2_6_22_OR_2_6_23)
	int err = 0;
#endif

	//
	//	Is skb fragmented ?
	//
	if (!(ip_hdr(frag_skb)->frag_off & htons(IP_MF | IP_OFFSET))) {
		return frag_skb;
	}		

	//
	//	include/net/ip.h defines enum ip_defrag_users { ... IP_DEFRAG_CONNTRACK_IN ... }; 
	//	it is used in fragmentation (comparing when looking for fragments in frag queue)
	//	max value in linux 2.6.23 is 6
	//
#define IP_DEFRAG_LINIPF	0xf

	//
	//	next piece of code has been seen in net/ipv4/netfilter/nf_conntrack_l3proto_ipv4.c!nf_ct_ipv4_gather_frags()
	//
	skb_orphan(frag_skb);

	local_bh_disable();

#if defined(LINUX_KERNEL_2_6_22_OR_2_6_23)

	defrag_skb = ip_defrag(frag_skb, IP_DEFRAG_LINIPF);

#else

	err = ip_defrag(frag_skb, IP_DEFRAG_LINIPF);

	//
	//	success, on error it will return EINPROGRESS (-115)
	// 
	if (err == 0) {
		defrag_skb = frag_skb;
	}

#endif

	local_bh_enable();

	if (!IS_PTR_VALID(defrag_skb)) {
		TRACE(LOG_DBG, "%s: ip_defrag(): skb=%p queued.", __FUNC__, frag_skb);			
		return NULL;
	}

	//
	//	Generate a checksum for an IP datagram
	//
	//ip_send_check(ip_hdr(defrag_skb));			

#if 0

	frag_skb = defrag_skb;

	while (frag_skb) {

		TRACE(LOG_INFO, "%s: frag skb=%p, head_len=%d, len=%d, data_len=%d, frag_list=%p, nr_frags=%d, ip=%p, data=%p, next=%p, prev=%p, nonlinear=%d",
			__FUNC__, frag_skb, skb_headlen(frag_skb), frag_skb->len, frag_skb->data_len, skb_shinfo(frag_skb)->frag_list, skb_shinfo(frag_skb)->nr_frags,
			ip_hdr(frag_skb), frag_skb->data, frag_skb->next, frag_skb->prev, skb_is_nonlinear(frag_skb));

		dump_ip_packet((u_int8_t*) ip_hdr(frag_skb), skb_headlen(frag_skb));

		frag_skb = ((frag_skb == defrag_skb) ? (skb_shinfo(frag_skb)->frag_list) : (frag_skb->next));
	}

#endif

	return defrag_skb;
}

#if !defined (USE_SCATTER_GATHER)

/**
 *	ip_frag_copy_fragments
 */
#undef __FUNC__
#define __FUNC__ "ip_frag_copy_fragments()"
static inline int 
ip_frag_copy_fragments(
	struct sk_buff *skb,
	char *buf_u,
	int buf_len,
	int offset,
	int *foff
	)
{
	int old_foff_for_log = *foff;
	int flen = skb_headlen(skb);
	
	int copy_offset = offset - (*foff);
	int copy_len = 0;
	
	if (copy_offset > 0) {
	
		if (copy_offset < flen) {
			copy_len = flen - copy_offset;
			(*foff) += copy_offset;
		}else {
			(*foff) += flen;
		}

	} else {
		
		copy_len = buf_len;

		if (copy_len > flen) {
			copy_len = flen;
		}
	}
	
	TRACE(LOG_DBG, "%s: offset = %d, foff = %d (%d), flen = %d, copy_len = %d",
		__FUNC__, offset, *foff, old_foff_for_log, flen, copy_len);

	if (copy_len > 0) {
		copy_to_user(buf_u + (*foff), skb->data, copy_len);
		(*foff) += copy_len;
	}
	
	return copy_len;
}

/**
 *	ip_frag_copy_packet_data
 */
#undef __FUNC__
#define __FUNC__ "ip_frag_copy_packet_data()"
int 
ip_frag_copy_packet_data(	
	struct sk_buff *skb,
	char *buf_u,
	int buf_len,
	int offset	
	)
{			 
	int bytes_up = 0;

	if (buf_len < 20) {		
		return bytes_up;
	}

	if (!IS_PTR_VALID(skb_shinfo(skb)->frag_list)) {
		
		//
		//	Non-fragmented traffic.
		//
		bytes_up = ((skb->len <= buf_len) ? (skb->len) : (buf_len));
		copy_to_user(buf_u + offset, (u_int8_t*)ip_hdr(skb) + offset, bytes_up - offset);
		
	} else {
		int copied = offset;		
		struct sk_buff *frag_skb = skb;
		int foff = 0;

#error "--- Error in implementation of ip_frag_copy_packet_data --- "

		//
		//	ToDo:	we have to add IP header first and then append data as in old linux wrdrv
		//		in long ICMP packet (2048) we can see double IP headers now :-(
		//				
		while (frag_skb) {

			copied += ip_frag_copy_fragments(frag_skb, buf_u, buf_len - copied, offset, &foff);

			TRACE(LOG_DBG, "%s: copied=%d", __FUNC__, copied);

			if (copied >= buf_len) {
				break;
			}

			frag_skb = ((frag_skb == skb) ? (skb_shinfo(frag_skb)->frag_list) : (frag_skb->next));
		}		

		bytes_up = copied;
	}

	return bytes_up;
}

/**
 *	ip_frag_set_iphdr_data_16
 */
#undef __FUNC__
#define __FUNC__ "ip_frag_set_iphdr_data_16()"
inline void 
ip_frag_set_iphdr_data_16(
	struct iphdr *iph,
	unsigned short *ptr,
	unsigned short data
	)
{
	if (data != *ptr) {
		//adjust_checksum(&iph->check, ptr, &data, 2);
		*ptr = data;
	}
}

/**
 *	ip_frag_update_fragment_header
 *
 *  Updates IP header of one fragment, preserves tot_len and frag_off fields
 *  and leaves correct checksum.
 */
#undef __FUNC__
#define __FUNC__ "ip_frag_update_fragment_header()"
static inline void
ip_frag_update_fragment_header(
	struct sk_buff *skb,
	struct iphdr *iph
	)
{
	struct iphdr *fiph = ip_hdr(skb);
	unsigned short foff = 0;
	unsigned short tlen = 0;

	foff = fiph->frag_off;
	tlen = fiph->tot_len;
	
	memcpy(fiph, iph, 20);

	ip_frag_set_iphdr_data_16(fiph, &fiph->frag_off, foff);
	ip_frag_set_iphdr_data_16(fiph, &fiph->tot_len, tlen);
}

/**
 *	ip_frag_update_fragment_data
 *
 *  Copies one fragment of IP traffic from userspace buffer.
 */
#undef __FUNC__
#define __FUNC__ "ip_frag_update_fragment_data()"
static inline int 
ip_frag_update_fragment_data(
	struct sk_buff *skb,
	char *buf_u,
	int len,
	int *foff
	)
{
	int flen = ((IS_PTR_VALID(skb_shinfo(skb)->frag_list)) ? (skb_headlen(skb) - 20) : (skb_headlen(skb)));
	int old_flen_for_log = flen;
	u_int8_t *dst = (u_int8_t*)ip_hdr(skb) + 20;
		
	if (flen > len) {
		flen = len;
	}

	TRACE(LOG_DBG, "%s: foff = %d, flen = %d (%d), len = %d", __FUNC__, *foff, flen, old_flen_for_log, len);

	if (flen > 0) {
		copy_from_user(dst, buf_u + (*foff), flen);
		(*foff) += flen;
	}

	return flen;
}

/**
 *	ip_frag_update_packet_data
 */
#undef __FUNC__
#define __FUNC__ "ip_frag_update_packet_data()"
void 
ip_frag_update_packet_data(
	struct sk_buff *skb,
	char *buf_u,
	int len
	)
{
	if (len < 20) {
		return;
	}

	if (!IS_PTR_VALID(skb_shinfo(skb)->frag_list)) {

		copy_from_user(ip_hdr(skb), buf_u, len);
	
	} else {
		int foff = 20;
		int updated = 20;
		struct iphdr iph;
		struct sk_buff *frag_skb = skb;

		copy_from_user(&iph, buf_u, 20);

		TRACE(LOG_DBG, "%s: updated=%d", __FUNC__, updated);

		while (frag_skb) {

			ip_frag_update_fragment_header(frag_skb, &iph);

			updated += ip_frag_update_fragment_data(frag_skb, buf_u, len - updated, &foff);

			TRACE(LOG_DBG, "%s: updated=%d", __FUNC__, updated);

			frag_skb = ((frag_skb == skb) ? (skb_shinfo(frag_skb)->frag_list) : (frag_skb->next));
		}
	}
}

#endif // USE_SCATTER_GATHER


