/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"

//
//	DETAILED_QUEUE_USAGE_LOG
//
//#define DETAILED_QUEUE_USAGE_LOG

/**
 *	queue_enable
 */
#undef __FUNC__
#define __FUNC__ "queue_enable()"
void
queue_enable(
	struct ipf_queue *queue,
	int enable
	)
{
	acquire_lock(&queue->lock);
	queue->enabled = enable;
	release_lock(&queue->lock);
}

/**
 *	queue_cmp_by_id
 */ 
#undef __FUNC__
#define __FUNC__ "queue_cmp_by_id()"
int
queue_cmp_by_id(
	struct sk_buff *skb,
	void *id
	)
{	
	return ((skb == id) ? (0) : (1));
}

/**
 *	queue_len
 */
int
queue_len(
	struct ipf_queue *queue
	)
{
	int len = 0;

	acquire_lock(&queue->lock);	

	len = queue->queued_pkts;

	release_lock(&queue->lock);	

	return len;
}

/**
 *	queue_is_empty
 */
#undef __FUNC__
#define __FUNC__ "queue_is_empty()"
int
queue_is_empty(
	struct ipf_queue *queue
	)
{
	int ret = 0;

	acquire_lock(&queue->lock);	

	ret = ((queue->queued_pkts) ? (0) : (1));

	release_lock(&queue->lock);	

	return ret;
}

/**
 *	queue_insert   
 */ 
#undef __FUNC__
#define __FUNC__ "queue_insert()"
int
queue_insert(
	struct ipf_queue *queue,
	struct sk_buff *skb,
	int tail
	)
{	
	acquire_lock(&queue->lock);	

	if (!queue->enabled) {
		release_lock(&queue->lock);
		return -EINVAL;
	}	

	if (tail) {
		__skb_queue_tail(&queue->q, skb);
	} else {
		__skb_queue_head(&queue->q, skb);
	}		

	queue->queued_pkts++;

#if defined (DETAILED_QUEUE_USAGE_LOG) 	
	TRACE(LOG_DBG,"%s: skb=%p queued.", __FUNC__, skb);	
#endif

	release_lock(&queue->lock);

	return 0;
}

/**
 *	queue_remove
 */  
#undef __FUNC__
#define __FUNC__ "queue_remove()"
struct sk_buff*
queue_remove(
	struct ipf_queue *queue,
	void *data,
	ipf_cmpfn cmpfn
	)
{		
	struct sk_buff *skb = NULL;

	acquire_lock(&queue->lock);	
	
	skb = ((struct sk_buff *)&queue->q)->next;

	while (skb != (struct sk_buff *)&queue->q) {

		if (!cmpfn || cmpfn(skb, data) == 0) {		

			__skb_unlink(skb, &queue->q);

			queue->queued_pkts--;

			release_lock(&queue->lock);

#if defined (DETAILED_QUEUE_USAGE_LOG) 
			TRACE(LOG_DBG,"%s: skb=%p removed from queue.", __FUNC__, skb);
#endif
			return skb;
		}		

		skb = skb->next;
	}		

	release_lock(&queue->lock);

	return NULL;
}

/**
 *	queue_init
 */
#undef __FUNC__
#define __FUNC__ "queue_init()"
void
queue_init(
	struct ipf_queue *queue
	)
{	
	//
	//	ToDo: add initialized variable
	//
	skb_queue_head_init(&queue->q);

	alloc_lock(&queue->lock);
		
	queue->queued_pkts = 0;
	queue->enabled = 0;
}

/**
 *	queue_close
 */
#undef __FUNC__
#define __FUNC__ "queue_close()"
void
queue_close(
	struct ipf_queue *queue
	)
{	
	int len = queue_len(queue);

	if (len) {
		TRACE(LOG_WARN, "%s: ### %d buffers still in queue. ###", __FUNC__, len);
	}

	free_lock(&queue->lock);
}

