/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __QUEUE_H__
#define __QUEUE_H__

//
//	GET_CB
//
#define GET_CB(skb)	((struct ipf_skb_cb*)((skb)->cb))

//
//	NETWORK_PACKET
//
#define NETWORK_PACKET	0x1ff1

//
//	NET_DEV_PACKET
//
#define NET_DEV_PACKET	0x2ff2

//
//	ipf_skb_cb
//
struct ipf_skb_cb {
#if defined(LINUX_KERNEL_2_6_22_OR_2_6_23)
	struct nf_info *nf_data;		
#else
	struct nf_queue_entry *nf_data;
#endif
	u_int32_t copied_len;
	u_int32_t type;
	bool received;
	bool policy_routing;
};

//
//	ipf_cmpfn
//
typedef int (*ipf_cmpfn)(struct sk_buff*, void*);

//
//	ipf_queue
//
struct ipf_queue {
	struct sk_buff_head q;
	K_LOCK lock;
	int enabled;
	unsigned int queued_pkts;
};

/**
 *	queue_enable
 */
void
queue_enable(
	struct ipf_queue *queue,
	int enable
	);

/**
 *	queue_cmp_by_id
 */ 
int
queue_cmp_by_id(
	struct sk_buff *skb,
	void *id
	);

/**
 *	queue_len
 */
int
queue_len(
	struct ipf_queue *queue
	);

/**
 *	queue_is_empty
 */
int
queue_is_empty(
	struct ipf_queue *queue
	);

/**
 *	queue_insert   
 */ 
int
queue_insert(
	struct ipf_queue *queue,
	struct sk_buff *skb,
	int tail
	);

/**
 *	queue_remove
 */  
struct sk_buff*
queue_remove(
	struct ipf_queue *queue,
	void *data,
	ipf_cmpfn cmpfn
	);

/**
 *	queue_init
 */
void
queue_init(
	struct ipf_queue *queue
	);

/**
 *	queue_close
 */
void
queue_close(
	struct ipf_queue *queue
	);

#endif //__QUEUE_H__
