/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"

/**
 *	send
 */
#undef __FUNC__
#define __FUNC__ "send()"
void
send(
	struct sk_buff *skb,
#if defined(LINUX_KERNEL_2_6_22_OR_2_6_23)
	struct nf_info *nf_data,	
#else
	struct nf_queue_entry *nf_data,
#endif
	void *context
	)
{			
	if (IS_DEBUG_FLAG_SET()) {

		TRACE(LOG_DBG, "%s: skb=%p, len=%d, head_len=%d, dev='%s', frag_list=%p, nr_frags=%d, ip_summed=%d, mac=%s, "
			"mac_hdr=%p, mac_len=%d, data_len=%d, ip=%p, head=%p, data=%p, tail=%p, end=%p, frag_list=%p, next=%p, prev=%p",
			__FUNC__, skb, skb->len, skb_headlen(skb), ((skb->dev) ? (skb->dev->name) : ("???")), skb_shinfo(skb)->frag_list,
			skb_shinfo(skb)->nr_frags, skb->ip_summed, ((skb_mac_header_was_set(skb)) ? ("yes") : ("no")), skb_mac_header(skb),
			skb->mac_len, skb->data_len, ip_hdr(skb), skb->head, skb->data, skb->tail, skb->end, skb_shinfo(skb)->frag_list,
			skb->next, skb->prev);

		dump_ip_packet((u_int8_t*) ip_hdr(skb), skb_headlen(skb));	
	}

	(GET_CB(skb))->nf_data = nf_data;
	(GET_CB(skb))->copied_len = 0;
	(GET_CB(skb))->type = NETWORK_PACKET;
	(GET_CB(skb))->received = false;	
	(GET_CB(skb))->policy_routing = false;	

	if (queue_insert(&g_main_queue, skb, 1)) {
		TRACE(LOG_WARN, "%s: queue_insert(): can't insert skb=%p into main queue.", __FUNC__, skb);
		nf_reinject(nf_data, NF_DROP);
		return;
	}

#if 0

	if (!(skb = queue_remove(&g_main_queue, skb, queue_cmp_by_id))) {					
		TRACE(LOG_WARN, "%s: queue_remove(): can't remove skb with id=%p from main queue.", __FUNC__, skb);				
		return;
	}		

	nf_reinject(nf_data, NF_ACCEPT);

#else

	io_dev_wakeup();

#endif			
}

