/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"

//
//	gs_socket
//
struct socket *gs_socket = NULL;

/**
 *	socket_init
 */
#undef __FUNC__
#define __FUNC__ "socket_init()"
int
socket_init(	
	void 
	)
{
	int ret = 0;
	int one = 1;		

	ret = sock_create_kern(
			PF_INET, 
			SOCK_RAW,
			IPPROTO_RAW,
			&gs_socket
			);	

	if (ret < 0) {
		TRACE(LOG_ERR, "%s: sock_create_kern(): ret=%d", __FUNC__, ret);
		return ret;
	}	
		
	ret = kernel_setsockopt(
			gs_socket,
			IPPROTO_IP,
			IP_HDRINCL,
			(char *) &one,
			sizeof(one)
			);

	if (ret < 0) {
		TRACE(LOG_ERR, "%s:  kernel_setsockopt(): ret=%d", __FUNC__, ret);				
		return ret;
	}

	TRACE(LOG_INFO, "%s: raw socket created.", __FUNC__);

	return ret;
}

/**
 *	socket_close
 */
#undef __FUNC__
#define __FUNC__ "socket_close()"
void
socket_close(	
	void 
	)
{
	if (IS_PTR_VALID(gs_socket)) {

		sock_release(gs_socket);
		gs_socket = NULL;

		TRACE(LOG_INFO, "%s: raw socket closed.", __FUNC__);
	}
}

/**
 *	socket_send_ip
 */
#undef __FUNC__
#define __FUNC__ "socket_send_ip()"
int
socket_send_ip(	
	unsigned char *buffer,
	int len,
	u_int32_t ip
	)
{
	struct msghdr msg;
	struct iovec iov;
	struct sockaddr_in addr;	
	int ret = 0;

	if (!IS_PTR_VALID(gs_socket)) {
		TRACE(LOG_WARN, "%s: raw socket is not created ???", __FUNC__);
		return -EINVAL;
	}
	
	addr.sin_port = 0;
	addr.sin_family = AF_INET;	
	addr.sin_addr.s_addr = ip;

	msg.msg_name = (struct sockaddr*) &addr;
	msg.msg_namelen = sizeof(struct sockaddr);

	iov.iov_base = buffer;
	iov.iov_len = len;

	msg.msg_iov = &iov;
	msg.msg_iovlen = 1;

	msg.msg_control = NULL;
	msg.msg_controllen = 0;	
	msg.msg_flags = 0;

	ret = kernel_sendmsg(
			gs_socket,
			&msg,
			(struct kvec*)&iov,
			1,
			len
			);		
	
	if (ret < 0) {
		TRACE(LOG_WARN, "%s: kernel_sendmsg(): ret=%d", __FUNC__, ret);
	} 	

	return ret;
}

/**
 *	socket_is_our
 */
#undef __FUNC__
#define __FUNC__ "socket_is_our()"
bool
socket_is_our(	
	struct socket *s
	)
{
	return ((s == gs_socket) ? (true) : (false));
}

