/*
 * Kerio IP Filter library for userspace.
 * Copyright (C) Kerio Technologies, s.r.o.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __KIPF_API_H__
#define __KIPF_API_H__

#include "status.h"

#include "../../../../drivers/ipf/linux/defs.h"
#include "../../../../drivers/ipf/linux/ioctl.h"

#ifdef __cplusplus
extern "C"{
#endif 

/*
 *	kipf_open
 */
kipf_status kipf_open(void);

/*
 *	kipf_close
 */
kipf_status kipf_close(unsigned int rule);

/*
 *	kipf_read
 */
kipf_status kipf_read(READ_REQ *rreq);

/*
 *	kipf_write
 */
kipf_status kipf_write(WRITE_REQ *wreq);

/*
 *	kipf_cancel_read
 */
kipf_status kipf_cancel_read(void);

/*
 *	kipf_get_driver_info
 */
kipf_status kipf_get_driver_info(struct kipf_info *info);

/*
 *	kipf_complete_packet
 */
kipf_status kipf_complete_packet(struct kipf_packet *packet);

/*
 *	kipf_drop_packet
 */
kipf_status kipf_drop_packet(struct kipf_packet *packet);

/*
 *	kipf_accept_packet
 */
kipf_status kipf_accept_packet(struct kipf_packet *packet);

/*
 *	kipf_accept_packet_with_info
 */
kipf_status kipf_accept_packet_with_info(struct kipf_packet *packet, unsigned int modif_len, struct kipf_route_info *route_info);

/*
 *	kipf_send_outgoing_packet
 */
kipf_status kipf_send_outgoing_packet(unsigned char *pkt, unsigned int pkt_len);

/*
 *	kipf_send_outgoing_packet_by_dev (PPP - ok (eth hdr not needed), ETH - not implemented)
 */
kipf_status kipf_send_outgoing_packet_by_dev(unsigned char *pkt, unsigned int pkt_len, unsigned int ifindex);

/*
 *	kipf_send_outgoing_packet_by_next_hop (PPP - ok, ETH - ok (use route_pkt()) )
 */
kipf_status kipf_send_outgoing_packet_by_next_hop(unsigned char *pkt, unsigned int pkt_len, unsigned int next_hop);

/*
 *	kipf_send_incoming_packet
 */
kipf_status kipf_send_incoming_packet(unsigned char *pkt, unsigned int pkt_len);

/*
 *	kipf_send_incoming_packet_by_dev (PPP - ok (eth hdr not needed), ETH - ok (eth hdr built from params))
 */
kipf_status kipf_send_incoming_packet_by_dev(unsigned char *pkt, unsigned int pkt_len, unsigned int ifindex);

/*
 *	kipf_send_incoming_packet_by_next_hop (PPP - not implemented, ETH - not implemented)
 */
kipf_status kipf_send_incoming_packet_by_next_hop(unsigned char *pkt, unsigned int pkt_len, unsigned int next_hop);

#ifdef __cplusplus
}
#endif

#endif // __KIPF_API_H__
